\\ Copyright 2019, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
default(recover,0);
read("../vecper.gp");

\\-----------------------------------------------------------------------------

f_to_z_base4(f) =
{
  \\ print("f_to_z_base4() ",vecper_frac_digits(f,4));
  vecper_make_state
    ((state)-> \\ print("  "state);
     my(f=state[1],
        xrev=state[2],
        yrev=state[3]);
     \\ print("  at f="f" xrev="lift(xrev)" yrev="lift(yrev));
     (0 <= f && f <= 1) 
     || error("f_to_z_base4() oops, f out of range: ",f);
     
     f*=4;
     my(x=min(3,floor(f)));     \\ x digit 0 to 3
     \\ print("  for x f="f" to digit x="x);
     f-=x;

     f*=4;
     my(y=min(3,floor(f)));     \\ y digit 0 to 3
     \\ print("      y f="f" to digit y="y);
     f-=y;
     
     if(xrev, x=3-x);           \\ possible reverse x
     xrev += y;
     yrev += x;
     if(yrev, y=3-y);           \\ possible reverse y
     [x + y*I, [f,xrev,yrev]];
     ,
     [f,Mod(0,2),Mod(0,2)]);   \\ initial whole f and no reversal
}
{
  my(s=2);
  my(l=List([]));
  for(n=0,4^(2*s),
     my(f=n/4^(2*s),
        v=f_to_z_base4(f),
        z=vecper_frac_fromdigits(v,4););
     print("n="n" f="f" z="z" int "z*4^s"  vecper "v);
     listput(l,z));
  l=Vec(l);
  plothraw(real(l),imag(l),1);
  quit;
}


\\-----------------------------------------------------------------------------

f_to_z_ternary(f) =
{
  vecper_make_state((state)->
                    my(f=state[1],
                       xrev=state[2],
                       yrev=state[3]);
                    (0 <= f && f <= 1)
                      || error("f_to_z_ternary() oops, f out of range: ",f);

                    f*=3;
                    my(x=min(2,floor(f)));     \\ x digit 0 to 2
                    f-=x;
                    f*=3;
                    my(y=min(2,floor(f)));     \\ y digit 0 to 2
                    f-=y;

                    if(xrev, x=2-x);           \\ possible reverse x
                    xrev += y;
                    yrev += x;
                    if(yrev, y=2-y);           \\ possible reverse y
                    [x + y*I, [f,xrev,yrev]];
                    ,
                    [f,Mod(0,2),Mod(0,2)]);   \\ initial whole f, no reversal
}
f_to_z(f) = vecper_frac_fromdigits(f_to_z_ternary(f),3);

ix=vector(6,n,n--;2^n)
v=vector(ix[#ix]);
for(i=1,#ix, v[ix[i]]=1);
v=[v,[]]

len=6
f=sum(i=0,len,1/3^(2^i));
z=f_to_z(sum(i=0,len,1/3^(2^i)))*3^(2^(len-1))
digits(f*3^(2^len),3)
digits(real(z),3)
digits(imag(z),3)
quit


\\-----------------------------------------------------------------------------
